#include "audiolayerwidget.h"


AudioLayerWidget::AudioLayerWidget(QWidget *parent, QString name):
    QGroupBox(parent)
  , m_suspendResumeButton(0)
  , m_refreshGUI(new QTimer(this))
  , m_currentMedia("")
  , m_mediaLoaded(false)
{
    this->setTitle(name);
    QVBoxLayout *layout = new QVBoxLayout;

    QHBoxLayout *status = new QHBoxLayout;
    m_statusLabel = new QLabel;
    m_statusLabel->setText(STATUS_LABEL);
    m_statusValue = new QLabel;
    status->addWidget(m_statusLabel);
    status->addWidget(m_statusValue);
    m_loopCheck = new QCheckBox();
    m_loopCheckLabel = new QLabel;
    m_loopCheckLabel->setText("Loop");
    connect(m_loopCheck, SIGNAL(stateChanged(int)), this, SLOT(loopChanged(int)));
    status->addWidget(m_loopCheck);
    status->addWidget(m_loopCheckLabel);
    layout->addLayout(status);

    QHBoxLayout *folderLoaded = new QHBoxLayout;
    m_folderLabel = new QLabel;
    m_folderLabel->setText(FOLDER_LABEL);
    m_folderValue = new QLabel;
    m_folderValue->setMaximumWidth(200);
    folderLoaded->addWidget(m_folderLabel);
    folderLoaded->addWidget(m_folderValue);
    layout->addLayout(folderLoaded);

    QHBoxLayout *fileLoaded = new QHBoxLayout;
    m_fileLabel = new QLabel;
    m_fileLabel->setText(FILE_LABEL);
    m_fileValue = new QLabel;
    m_fileValue->setMaximumWidth(200);
    fileLoaded->addWidget(m_fileLabel);
    fileLoaded->addWidget(m_fileValue);
    layout->addLayout(fileLoaded);

    QGridLayout *volumeBox = new QGridLayout;
    m_volumeLabel = new QLabel;
    m_volumeLabel->setText(tr(VOLUME_LABEL));
    m_volumeSlider = new QSlider(Qt::Horizontal);
    m_volumeSlider->setMinimum(-100);
    m_volumeSlider->setMaximum(100);
    m_volumeSlider->setSingleStep(1);
    m_volumeIndicator = new QLabel;
    volumeBox->addWidget(m_volumeLabel, 0, 0);
    volumeBox->addWidget(m_volumeSlider, 0, 1);
    volumeBox->addWidget(m_volumeIndicator, 0, 2);
    connect(m_volumeSlider, SIGNAL(valueChanged(int)), this, SLOT(volumeChanged(int)));
    connect(m_volumeSlider, &QSlider::valueChanged, this, [=] () {
        m_volumeIndicator->setText(QString::number(m_volumeSlider->value()));
    });
    m_panLabel = new QLabel;
    m_panLabel->setText("Pan");
    m_panSlider = new QSlider(Qt::Horizontal);
    m_panSlider->setMinimum(0);
    m_panSlider->setMaximum(255);
    m_panSlider->setSingleStep(1);
    connect(m_panSlider, SIGNAL(valueChanged(int)), this, SLOT(panChanged(int)));
    volumeBox->addWidget(m_panLabel, 1, 0);
    volumeBox->addWidget(m_panSlider, 1, 1);
    m_pitchLabel = new QLabel;
    m_pitchLabel->setText("Pitch");
    m_pitchSlider = new QSlider(Qt::Horizontal);
    m_pitchSlider->setMinimum(0);
    m_pitchSlider->setMaximum(255);
    m_pitchSlider->setSingleStep(1);
    connect(m_pitchSlider, SIGNAL(valueChanged(int)), this, SLOT(pitchChanged(int)));
    volumeBox->addWidget(m_pitchLabel, 2, 0);
    volumeBox->addWidget(m_pitchSlider, 2, 1);
    layout->addLayout(volumeBox);

    QHBoxLayout *progressTime = new QHBoxLayout;
    m_progressTimeLabel = new QLabel;
    m_progressTimeLabel->setText(PROGRESS_TIME_LABEL);
    m_progressTime = new QTimeEdit;
    m_progressTime->text();
    m_progressTime->setDisplayFormat("h:mm:ss:zzz");
    m_progressTime->setReadOnly(true);
    m_progressTime->setButtonSymbols(QAbstractSpinBox::NoButtons);
    m_progressTime->setMaximumWidth(120);
    progressTime->addWidget(m_progressTimeLabel);
    progressTime->addWidget(m_progressTime);
    m_totalTimeLabel = new QLabel;
    m_totalTimeLabel->setText(TOTAL_TIME_LABEL);
    m_totalTimeValue = new QTimeEdit;
    m_totalTimeValue->setDisplayFormat("h:mm:ss:zzz");
    m_totalTimeValue->setReadOnly(true);
    m_totalTimeValue->setButtonSymbols(QAbstractSpinBox::NoButtons);
    m_totalTimeValue->setMaximumWidth(120);
    progressTime->addWidget(m_totalTimeLabel);
    progressTime->addWidget(m_totalTimeValue);
    layout->addLayout(progressTime);

    m_progressSlider = new QSlider(Qt::Horizontal);
    layout->addWidget(m_progressSlider);

    m_suspendResumeButton = new QPushButton(this);
    m_suspendResumeButton->setText(tr(SUSPEND_LABEL));
    connect(m_suspendResumeButton, SIGNAL(clicked()), SLOT(toggleSuspendResume()));
    layout->addWidget(m_suspendResumeButton);

    this->setLayout(layout);
    connect(m_refreshGUI, SIGNAL(timeout()), this, SLOT(refreshGUI()));
    m_refreshGUI->start(100);
}

AudioLayerWidget::~AudioLayerWidget()
{

}

void AudioLayerWidget::volumeChanged(int value)
{
    float result;

    if (m_mediaLoaded == false)
        return;
    result = ma_volume_linear_to_db(value);
    ma_sound_group_set_volume(&currentSound, result);
}

void AudioLayerWidget::panChanged(int value)
{
    float result;
    if (m_mediaLoaded == false)
        return;
    result = (value / 128.0) - 128;
    ma_sound_group_set_pan(&currentSound, result);
}

void AudioLayerWidget::pitchChanged(int value)
{
    float result;
    if (m_mediaLoaded == false)
        return;
    result = (value / 128.0) - 128;
    ma_sound_group_set_pitch(&currentSound, result);
}

void AudioLayerWidget::loopChanged(int value)
{
    if (m_mediaLoaded == false)
        return;
    ma_sound_set_looping(&currentSound, value);
}

void AudioLayerWidget::setVol(qreal vol)
{
    this->volumeChanged(vol);
    m_volumeSlider->blockSignals(true);
    m_volumeSlider->setValue(vol);
    m_volumeIndicator->setText(QString::number(vol));
    m_volumeSlider->blockSignals(false);
}

void AudioLayerWidget::setPan(qreal pan)
{
    this->panChanged(pan);
    m_panSlider->blockSignals(true);
    m_panSlider->setValue(pan);
    m_panSlider->blockSignals(false);
}

void AudioLayerWidget::setPitch(qreal pitch)
{
    this->pitchChanged(pitch);
    m_pitchSlider->blockSignals(true);
    m_pitchSlider->setValue(pitch);
    m_pitchSlider->blockSignals(false);
}

void AudioLayerWidget::loadMedia(QString file)
{
    ma_result result;
    ma_format *format = 0;
    ma_uint32 *channels = 0;
    ma_uint32 *sampleRate = 0;


    if (m_currentMedia.compare(file) == 0 ) {
        return;
    }
    if (!QFile::exists(file)) {
        qWarning("Can not access to file %s", file.toLatin1().constData());
        return;
    }
    ma_engine engine = AudioWidget::getInstance()->getEngine();
    if (currentSound.ownsDataSource == true)
    {
        ma_sound_uninit(&currentSound);
    }
    result = ma_sound_init_from_file(&engine, file.toLatin1(), MA_SOUND_FLAG_NO_SPATIALIZATION | MA_RESOURCE_MANAGER_DATA_SOURCE_FLAG_DECODE | MA_RESOURCE_MANAGER_DATA_SOURCE_FLAG_ASYNC | MA_RESOURCE_MANAGER_DATA_SOURCE_FLAG_STREAM, NULL, NULL, &currentSound);
    if (result != MA_SUCCESS) {
        qWarning("WARNING: Failed to load sound %s", file.toLatin1().constData());
        return;
    }
    m_currentMedia = file;
    float pLength = this->getDuration();
    result = ma_sound_get_data_format(&currentSound, format, channels, sampleRate, NULL, 0);
    if (result != MA_SUCCESS) {
        qWarning("WARNING: Failed to get data format %s", file.toLatin1().constData());
        return;
    }
    m_mediaLoaded = true;
    durationChanged(pLength * 1000);
    fileLoaded(file);
    // Display music informations
    std::cout << "File loaded: " << file.toLatin1().constData() << " : " << std::endl;
    std::cout << " " << pLength << " seconds";
    std::cout << " -- " << sampleRate << " samples/sec";
    std::cout << " -- " << format << " format";
    std::cout << " -- " << channels << " channels" << std::endl;
}

float AudioLayerWidget::getDuration()
{
    float pLength;

    if (m_mediaLoaded == false)
        return 0;
    ma_result result = ma_sound_get_length_in_seconds(&currentSound, &pLength);
    if (result != MA_SUCCESS) {
        qWarning("WARNING: Failed to get total duration %s", m_currentMedia.toLatin1().constData());
        return 0;
    }
    return pLength;
}

void AudioLayerWidget::fileLoaded(QString file)
{
    QStringList list = file.split("/");
    int size = list.size();
    if (size >= 2) {
       m_folderValue->setText(list.at(size - 2));
       m_fileValue->setText(list.at(size - 1));
    }
}

void AudioLayerWidget::durationChanged(qint64 dur)
{
    m_progressSlider->setMaximum(dur);
    m_totalTimeValue->setTime(QTime::fromMSecsSinceStartOfDay(dur));
}

void AudioLayerWidget::play(bool loop)
{
    if (m_mediaLoaded == false)
        return;
    ma_sound_set_looping(&currentSound, loop);
    ma_sound_start(&currentSound);
    m_loopCheck->blockSignals(true);
    m_loopCheck->setChecked(loop);
    m_loopCheck->blockSignals(false);
}

void AudioLayerWidget::pause()
{
    if (m_mediaLoaded == false)
        return;
    ma_sound_stop(&currentSound);
}

void AudioLayerWidget::stop()
{
    if (m_mediaLoaded == false)
       return;
    ma_sound_stop(&currentSound);
    ma_sound_seek_to_pcm_frame(&currentSound, 0);
    m_progressTime->setTime(QTime::fromMSecsSinceStartOfDay(0));
    m_statusValue->setText(STOP_LABEL);
    m_suspendResumeButton->setText(tr(STOP_LABEL));
    m_progressSlider->setValue(0);
}

void AudioLayerWidget::refreshGUI() {
    float progress;

    if (m_mediaLoaded == false)
        return;
    if (currentSound.ownsDataSource == 0)
        return;
    switch (ma_sound_is_playing(&currentSound)) {
      case true:
         ma_sound_get_cursor_in_seconds(&currentSound, &progress);
         progress *= 1000;
         m_progressSlider->setValue(progress);
         m_progressTime->setTime(QTime::fromMSecsSinceStartOfDay(progress));
         m_statusValue->setText(PLAY_LABEL);
         m_suspendResumeButton->setText(tr(SUSPEND_LABEL));
         break;
      case false:
         m_statusValue->setText(PAUSE_LABEL);
         m_suspendResumeButton->setText(tr(RESUME_LABEL));
         break;
    }
}

void AudioLayerWidget::toggleSuspendResume()
{
    if (m_mediaLoaded == false)
        return;
    if (ma_sound_is_playing(&currentSound)) {
        this->pause();
    } else {
        if (ma_sound_at_end(&currentSound))
            ma_sound_seek_to_pcm_frame(&currentSound, 0);
        ma_sound_start(&currentSound);
    }
}
