#ifndef AUDIOLAYERWIDGET_H
#define AUDIOLAYERWIDGET_H

#include <iostream>

#include <QDebug>
#include <QFile>
#include <QTime>
#include <QByteArray>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QLabel>
#include <QPushButton>
#include <QSlider>
#include <QTimer>
#include <QTimeEdit>
#include <QGroupBox>
#include <QCheckBox>

#include "defines.h"
#include "audiowidget.h"
#include "miniaudio.h"

class AudioLayerWidget : public QGroupBox
{
    Q_OBJECT
    
public:

    explicit AudioLayerWidget(QWidget *parent = 0, QString name = "Layer");
    ~AudioLayerWidget();

    /**
     * @brief load a new media file
     * @param file name with full path
     */
    void loadMedia(QString file);
    void play(bool loop);
    void stop();
    void pause();

    /**
     * @brief set the Volume
     * @param vol volume range 0 -100
     */
    void setVol(qreal vol);
    void resume();
    void setPan(qreal pan);
    void setPitch(qreal pitch);
    void setLoop(bool on);
    //void setEntryPoint(qreal entry);
    //void setEndPoint(qreal end);

public slots:

    /**
     * @brief connected with the button
     */
    void toggleSuspendResume();

    /**
     * @brief Connected with the slider
     * @param volume 0 -100 range
     */
    void volumeChanged(int vol);
    void panChanged(int vol);
    void pitchChanged(int vol);
    void loopChanged(int vol);

private:

    QPushButton *m_suspendResumeButton;

    QLabel *m_statusLabel;
    QLabel * m_statusValue;

    QLabel *m_volumeLabel;
    QSlider *m_volumeSlider;
    QLabel *m_volumeIndicator;

    QLabel *m_panLabel;
    QSlider *m_panSlider;

    QLabel *m_pitchLabel;
    QSlider *m_pitchSlider;

    QLabel *m_loopCheckLabel;
    QCheckBox *m_loopCheck;

    QLabel * m_progressLabel;
    QSlider *m_progressSlider;

    QLabel *m_progressTimeLabel;
    QTimeEdit *m_progressTime;

    QLabel *m_totalTimeLabel;
    QTimeEdit *m_totalTimeValue;

    QLabel *m_fileLabel;
    QLabel *m_fileValue;

    QLabel * m_folderLabel;
    QLabel * m_folderValue;

    QTimer *m_refreshGUI;
    QString m_currentMedia;
    ma_sound currentSound;
    ma_bool8 m_mediaLoaded;

    float getDuration();

private slots:

    /**
     * @brief Update the GUI elements with the name of the new file loaded
     * @param file
     */
    void fileLoaded(QString file);

    /**
     * @brief Update the GUI elements with the duration of the new file loaded
     * @param dur The duration of the track in miliseconds
     */
    void durationChanged(qint64 dur);

    /**
     * @brief Update the variable elements in GUI
     */
    void refreshGUI();
};

#endif // AUDIOLAYERWIDGET_H
