/*
   Libre Media Server Audio - An Open source Media Server for arts and performing.
   (c) Criptomart - Santiago Noreña 2012-2024 <lms@criptomart.net>
   https://git.criptomart.net/libremediaserver

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef LIBREMEDIASERVERAUDIO_H
#define LIBREMEDIASERVERAUDIO_H

#include <QMainWindow>
#include <QDockWidget>
#include <QFile>
#include <QFileInfo>
#include <QFileDialog>
#include <QTextStream>
#include <QWebView>
#include <QApplication>
#include <QVBoxLayout>
#include <QTextEdit>

#include "audiowidget.h"
#include "medialibrary.h"
#include "olathread.h"
#include "settings.h"
#include "audiomasterwidget.h"
#include "settingsdialog.h"
#include "defines.h"

#include "ui_libremediaserver-audio.h"

class QMenu;

class libreMediaServerAudio : public QMainWindow
{
    Q_OBJECT

public:

    libreMediaServerAudio (QStringList args, QWidget *parent = 0);
    virtual ~libreMediaServerAudio();

    Ui::LibreMediaServerAudio ui;

//    static QTextEdit *textEdit; // Terminal de feedback

protected:

private:

//   void MessageHandler(QtMsgType type, const QMessageLogContext &logcontext, const QString &msg);
   AudioMasterWidget *amw;

   olaThread *ola;

   void open_start();
   void save_finish();
   void open(QFile *file);
   void save(QFile *file);

public slots:
   void audioDeviceChanged(int id);

private slots:

   /**
     * @brief Shows the OLA web setup page
     */
    void olasetup();

    /**
     * @brief Parser for new dmx data arriving
     * @param layer
     * @param channel
     * @param value
     */
    void dmxInput(int layer, int channel, int value);

    // Menu File
    /**
     * @brief REad from dis a configuration file
     */
    void openFile();
    /**
     * @brief Write to disk a configuration file
     */
    void saveFile();
    /**
     * @brief OPen the settings dialog
     */
    void settings();
};

#endif // LIBREMEDIASERVERAUDIO_H
